@@ -289,6 +289,8 @@ sub _make_instance_info
 
 sub _make_unique_instance_info
 {
+    # This must be kept in sync with cleanup_leftovers, which expects
+    # to be able to recognise instance directories by name for cleanup.
     if (!defined $stamp)
     {
         $stamp = to_iso8601(DateTime->now);
@@ -383,13 +385,23 @@ sub cleanup_leftovers
     my @dirs;
     while (my $e = readdir(ROOT))
     {
-        push(@dirs, $e) if ($e =~ m/^[0-9]{6}([A-Z]|)[0-9]{1,}$/);
+        # This must be kept in sync with _make_unique_instance_info,
+        # which is what names and creates these directories.
+        my $basedirpat = qr{
+            \d{6}               # UTC timestamp as HHMMSS
+            (?:[0-9A-F]{2,})?   # optional worker ID as 2+ hex digits
+            [0-9A-F]{2,}        # unique number as 2+ hex digits
+        }ax;
+
+        push(@dirs, $e) if $e =~ m/$basedirpat/;
     }
     closedir ROOT;
 
     map
     {
-        xlog "Cleaning up old basedir $rootdir/$_";
+        if (get_verbose) {
+            xlog "Cleaning up old basedir $rootdir/$_";
+        }
         rmtree "$rootdir/$_";
     } @dirs;
 }
@@ -590,8 +602,10 @@ sub _list_pid_files
     my ($self) = @_;
 
     my $rundir = $self->{basedir} . "/run";
-    opendir(RUNDIR, $rundir)
-        or die "Cannot open run directory $rundir: $!";
+    if (!opendir(RUNDIR, $rundir)) {
+        return if $!{ENOENT}; # no run dir? never started
+        die "Cannot open run directory $rundir: $!";
+    }
 
     my @pidfiles;
     while ($_ = readdir(RUNDIR))
@@ -1023,7 +1037,8 @@ sub set_smtpd {
     }
 }
 
-sub _start_smtpd {
+sub _start_smtpd
+{
     my ($self) = @_;
 
     my $basedir = $self->{basedir};
@@ -1058,19 +1073,28 @@ sub _start_smtpd {
     }
 
     # Parent process.
-    $self->{smtphost} = $host . ':' . $port;
 
-    # XXX give the child a moment to actually start up before we start
-    # XXX assuming it has
+    # give the child a moment to actually start up
     sleep 1;
 
-    xlog "started smtpd as $smtppid";
-    push @{$self->{_shutdowncallbacks}}, sub {
-        local *__ANON__ = "kill_smtpd";
-        my $self = shift;
-        xlog "killing smtpd $smtppid";
-        kill(15, $smtppid);
-        waitpid($smtppid, 0);
+    # and then make sure it did!
+    my $waitstatus = waitpid($smtppid, WNOHANG);
+    if ($waitstatus == 0) {
+        $self->{smtphost} = $host . ':' . $port;
+
+        xlog "started smtpd as $smtppid";
+        push @{$self->{_shutdowncallbacks}}, sub {
+            local *__ANON__ = "kill_smtpd";
+            my $self = shift;
+            xlog "killing smtpd $smtppid";
+            kill(15, $smtppid);
+            waitpid($smtppid, 0);
+        };
+    }
+    else {
+        # child process already exited, something has gone wrong
+        Cassandane::PortManager::free($port);
+        die "smtpd with pid=$smtppid failed to start";
     }
 }
 
@@ -1467,13 +1491,22 @@ sub send_sighup
     return 1;
 }
 
+#
+# n.b. If you are stopping the instance intending to restart it again later,
+# you must set:
+#     $instance->{'re_use_dir'} => 1
+# before restarting, otherwise it will wipe and re-initialise its basedir
+# during startup, probably ruining whatever you were trying to do.  It
+# will still use the same directory name though, so it won't be obvious
+# from the logs that this is happening!
+#
 sub stop
 {
     my ($self) = @_;
 
     $self->_init_basedir_and_name();
 
-    return if ($self->{_stopped});
+    return if ($self->{_stopped} || !$self->{_started});
     $self->{_stopped} = 1;
 
     my @errors;
@@ -1518,18 +1551,6 @@ sub stop
     return @errors;
 }
 
-sub cleanup
-{
-    my ($self) = @_;
-
-    if (Cassandane::Cassini->instance()->bool_val('cassandane', 'cleanup'))
-    {
-        # Remove all on-disk traces of this instance
-        xlog "Cleaning up basedir " . $self->{basedir};
-        rmtree $self->{basedir};
-    }
-}
-
 sub DESTROY
 {
     my ($self) = @_;
