@@ -150,9 +150,15 @@ last command will be used if one is not specified.
 
 Show help for C<command> or all commands.
 
+=item C<getmetadata> [I<mailbox>]
+
+=item C<getmd> [I<mailbox>]
+
+Display mailbox/server metadata
+
 =item C<info> [I<mailbox>]
 
-Display the mailbox/server metadata.
+Display the mailbox/server annotations.
 
 =item C<listaclmailbox> I<mailbox>
 
@@ -382,6 +388,37 @@ them (unless overridden by a mailbox annotation).
 
 =back
 
+=item C<setmetadata> [--private] mailbox [I<annotation>] I<value>
+
+=item C<setmd> [--private] mailbox [I<annotation>] I<value>
+
+Set metadata on mailbox, where <annotation> is one of
+[comment|expire|news2mail|pop3showafter|sharedseen|sieve|specialuse|
+squat|/<explicit annotation>].
+
+Note that flags with a leading backslash must be escaped with an
+additional backslash.  For example:
+
+=over 4
+
+  setmetadata --private Spam specialuse "\\Junk"
+
+=back
+
+Note, too, that "private" annotations are private to the user currently
+authenticated as, not necessarily the owner of the mailbox.  To set
+annotations for another user you must authorize as that user.
+
+In addition to the use of optional flag B<--private>, one may use a more
+explicit syntax, prefixing the annotation with '/shared/' or '/private/'
+as in this example:
+
+=over 4
+
+  setmetadata Spam /private/specialuse "\\Junk"
+
+=back
+
 =item C<setquota> I<root> I<resource> I<value> [I<resource> I<value> ...]
 
 =item C<sq> I<root> I<resource> I<value> [I<resource> I<value> ...]
